<?php

declare(strict_types=1);

namespace Drupal\Tests\private_message\Functional;

use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\private_message\Traits\PrivateMessageTestTrait;

/**
 * Tests for the Private Message Recipients.
 *
 * @group private_message
 */
class PrivateMessageThreadMembersTest extends BrowserTestBase {

  use PrivateMessageTestTrait;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['private_message_test'];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->attachFullNameField();
    $this->createTestingUsers(3);
  }

  /**
   * Tests the thread members.
   */
  public function testThreadMembers(): void {
    $this->drupalLogin($this->users['a']);

    $this->drupalGet('/private-message/create');
    $this->assertSession()->statusCodeEquals(200);
    $this->getSession()->getPage()->pressButton('edit-members-add-more');
    $this->assertSession()->fieldExists('members[1][target_id]');
    $this->submitForm([
      'members[0][target_id]' => $this->users['b']->getAccountName(),
      'members[1][target_id]' => $this->users['c']->getAccountName(),
      'message[0][value]' => $this->getRandomGenerator()->sentences(5),
    ], 'Send');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementTextContains('css', '.private-message-recipients article:nth-of-type(1) .username', $this->users['b']->getDisplayName());
    $this->assertSession()->elementTextContains('css', '.private-message-recipients article:nth-of-type(2) .username', $this->users['c']->getDisplayName());
    $this->drupalLogin($this->users['b']);
    $this->drupalGet('private-message/create');
    $this->assertSession()->statusCodeEquals(200);
    $this->submitForm([
      'members[0][target_id]' => $this->users['c']->getAccountName(),
      'message[0][value]' => $this->getRandomGenerator()->sentences(5),
    ], 'Send');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementTextContains('css', '.private-message-recipients .username', $this->users['c']->getDisplayName());

    $this->drupalLogin($this->users['c']);
    $this->drupalGet('private-message/create');
    $this->assertSession()->statusCodeEquals(200);
    $this->submitForm([
      'members[0][target_id]' => $this->users['a']->getAccountName(),
      'message[0][value]' => $this->getRandomGenerator()->sentences(5),
    ], 'Send');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementTextContains('css', '.private-message-recipients .username', $this->users['a']->getDisplayName());
  }

  /**
   * Tests the autocomplete tags widget.
   */
  public function testThreadMembersWithTagsStyleWidget(): void {
    $form_display = EntityFormDisplay::load('private_message_thread.private_message_thread.default');
    $members = $form_display->getComponent('members');
    $members['type'] = 'entity_reference_autocomplete_tags';
    $form_display->setComponent('members', $members);
    $form_display->save();

    $this->drupalLogin($this->users['a']);

    $this->drupalGet('/private-message/create');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->fieldExists('members[target_id]');
    $this->submitForm([
      'members[target_id]' => $this->users['b']->getAccountName() . ' (' . $this->users['b']->id() . '), ' . $this->users['c']->getDisplayName() . ' (' . $this->users['c']->id() . ')',
      'message[0][value]' => $this->getRandomGenerator()->sentences(5),
    ], 'Send');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementTextContains('css', '.private-message-recipients article:nth-of-type(1) .username', $this->users['b']->getDisplayName());
    $this->assertSession()->elementTextContains('css', '.private-message-recipients article:nth-of-type(2) .username', $this->users['c']->getDisplayName());
  }

  /**
   * Tests the thread members.
   */
  public function testThreadUniqueness(): void {
    $this->drupalLogin($this->users['a']);

    $this->drupalGet('/private-message/create');
    $this->assertSession()->statusCodeEquals(200);
    $this->submitForm([
      'members[0][target_id]' => $this->users['b']->getAccountName(),
      'message[0][value]' => $this->getRandomGenerator()->sentences(5),
    ], 'Send');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementTextContains('css', '.private-message-recipients article:nth-of-type(1) .username', $this->users['b']->getDisplayName());

    // Assure that when we create another thread, with an extra user, this
    // group get their own, unique thread.
    $this->drupalGet('/private-message/create');
    $this->assertSession()->statusCodeEquals(200);
    $this->getSession()->getPage()->pressButton('edit-members-add-more');
    $this->assertSession()->fieldExists('members[1][target_id]');
    $this->submitForm([
      'members[0][target_id]' => $this->users['b']->getAccountName(),
      'members[1][target_id]' => $this->users['c']->getAccountName(),
      'message[0][value]' => $this->getRandomGenerator()->sentences(5),
    ], 'Send');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->elementTextContains('css', '.private-message-recipients article:nth-of-type(1) .username', $this->users['b']->getDisplayName());
    $this->assertSession()->elementTextContains('css', '.private-message-recipients article:nth-of-type(2) .username', $this->users['c']->getDisplayName());
  }

  /**
   * Regression test for multiple add more clicks.
   */
  public function testMultipleAddMoreClicks(): void {
    $this->drupalLogin($this->users['a']);

    $this->drupalGet('/private-message/create');

    $this->getSession()->getPage()->pressButton('edit-members-add-more');
    $this->assertSession()->statusCodeEquals(200);
    $this->getSession()->getPage()->pressButton('edit-members-add-more');
    $this->assertSession()->statusCodeEquals(200);
  }

}
